
	LIST P=18LF25K50
	#include <p18lf25k50.inc>
	#include <macros.inc>

; global farm
; extern Flag, KeyEdge, Rotor0, Rotor1, Rotor2, Rotor3, tx.byte, tx.ascnum, GPreg
; extern Buffer, Buffer2, BufferPause, rnd, Ma0, Ma1, Ma2, Ma3, Mc0, Mc1, Mc2, Mc3
; extern RXptr, RXpatience, RXFlag, MYserial, DispBuf, PowerOff, PowerCount

access	equ	0
banked	equ	1
rnd		equ	Offset+0x0004

;***************************************************************************************************
;	KERNEL RAM DEFINITIONS	(don't modify, or keep it equal as in Kernel.asm file!)					
;***************************************************************************************************

	CBLOCK		0

KeyEdge			; bits 3,2,1,0 are flags, set when keys 3,2,1,0 are JUST pressed
				; must be cleared in user routine
				; bit 0 set if key 0 (INT) just pressed
				; bit 1 set if key 1 (left) just pressed
				; bit 2 set if key 2 (up) just pressed
				; bit 3 set if key 3 (down) just pressed
				; bit 4 set if key 4 (right) just pressed

Rotor0			; used for key 0 (INT) debouncer
Rotor1			; used for key 1 (left) debouncer
Rotor2			; used for key 2 (up) debouncer
Rotor3			; used for key 3 (down) debouncer
Rotor4			; used for key 4 (right) debouncer

Flag			; bit 0 set = pause mode
				; bit 1 set = Timer interrupt (1200 Hz) handshaking (set only)
				; bit 2 set = full scan (150 Hz) handshaking (set only)
				; bit 3 set = EEPROM buffer function disabled
				; bit 4 set = INT in second cycle (OFF period)
				; bit 5 set = Only two steps for key 0, without pause
				; bit 6 set = Display message received
				; bit 7 set = column cathode

RXFlag			; bit 0 set = RX enable
				; bit 1 set = RX header reception in progress
				; bit 2 set = RX in progress
				; bit 3 set = RX message recived, flag to write terminator to EEPROM after timeout
				; bit 4 set = /2 countdown (1200 Hz ---> 600 Hz)
				; bit 5 set = 
				; bit 6 set = 
				; bit 7 set = 

Brightness		; display ON time, 0...15, used for dimming (15 = max brightness)
GPreg			; general purpose register
BitMask			; 10000000...00000001, shift right (CA) or 0...7 row counter (CC)
BitMask2		; 11111110...01111111, shift left
T0period		; total (OFF+ON) period for T0
InnerInt		; inner loop counter
OuterInt		; outer loop counter
OutPlusInt		; one more outer loop counter
RXptr			; low RXD buffer pointer, high is always =6
RXpatience		; Patience counter, preset when byte received, count down
PowerOff		; Auto Power Off period (*6sec)
PowerCount		; Auto Power Off count down
Uniform:2		; 150Hz freerunning uniform count up for 6 sec timing
RXserial:2		; received serial number BINARY, ready for comparison
MYserial:2		; serial number from ROM addr 0x100E
FSR0temp:2		; temporary FSR0 during INT

;***********************************************************
;	DEMO RAM DEFINITIONS									
;***********************************************************

FlagDemo		; bit 0 set = long pause (display only)
				; bit 1 set = next char immediate (display only)
				; bit 2 set = showing final result (tetris)
				; bit 3 set = key action, bit 3 clr = no action or vertical action
Inner 			; inner loop counter (used outside int routine)
Outer			; outer loop counter (used outside int routine)
Tempw			; Temporary W storage
Speed			; 150Hz divider, global timing
Block			; current block number 0 to 6, 0=I, 1=J, 2=L, 3=O, 4=S, 5=Z, 6=T
CurrentBlock:4	; bitmapped current block in current angle and current BlockX, bits 7-0
BlockX			; Current block X position, 1 to 8
BlockY			; Current block Y position, 0 (which is -1) to 15 (which is +14)
BlockA			; Current block angle, 0 to 3
Temp.BlockX 	; Temporary, in case the command was forbidden
Temp.BlockA 	; Temporary, in case the command was forbidden
Block.count 	; block counter, for speed increasing
Speed.index 	; Speed, 0 = slowest, 15 = fastest
liqflag			; bit 0 set = move X, bit 1 set = move Y
Xtest			; X pos for test if the pixel is occupied
Ytest			; Y pos for test if the pixel is occupied
WhoAmI			; Accelerometer Device ID (0x68 or 0xE5, otherwise 0x00)
DispSpeed		; Display period (number of display refresh steps between disp scrolls)
DispPtr			; Display text low pointer (high pointer is always 6)
ColCount		; Display column step counter
Score:2			; Tetris score (binary)
Bin:2			; lo,hi: Binary number for bin2bcd conversion
Bcd:3			; 7654 tens 3210 ones, 7654 thousands 3210 hundreds...
AccX:2			; Accelerometer X
AccY:2			; Accelerometer Y
AccZ:2			; Accelerometer Z

	ENDC

DispBuf		equ	0x600

Buffer		equ	0x700	; display buffer, upper row first, bit 7 = left column, bit set = LED on
Buffer2		equ	0x710	; aux buffer (not displayed by interrupt display refresh)
BufferPause	equ	0x720	; aux buffer (displayed only during pause)
Ma0			equ	0x730	; rnd seed
Ma1			equ	0x731
Ma2			equ	0x732
Ma3			equ	0x733
Mc0			equ	0x734	; rnd acc
Mc1			equ	0x735
Mc2			equ	0x736
Mc3			equ	0x737

;*******************************************************************************
;						D E M O   F A R M	 									
;*******************************************************************************

	org		Offset+0x0800
	goto	Offset+0x0008

	org		Offset+0x0806
	goto	Offset+0x0008

	org		Offset+0x080C
farm
	bcf		RXFlag,0,access	; disable RX
	bsf		Flag,5,access	; bit 5 set = Only two steps for key 0, without pause

	movlw	low (inimes)
	movwf	TBLPTRL
	movlw	high (inimes)
	movwf	TBLPTRH
	lfsr	FSR0,Buffer
	call	print.rom		; move .16 bytes from TBLPTR to FSR0

test.keys
	btfsc	KeyEdge,1,access
	goto	accelerometer
	btfsc	KeyEdge,2,access
	goto	display
	btfsc	KeyEdge,4,access
	goto	tetris
	btfsc	KeyEdge,3,access
	goto	id
	bra		test.keys

;																				
move2up				; shift buffer2 up
	movff	Buffer2+.1,Buffer2+.0
	movff	Buffer2+.2,Buffer2+.1
	movff	Buffer2+.3,Buffer2+.2
	movff	Buffer2+.4,Buffer2+.3
	movff	Buffer2+.5,Buffer2+.4
	movff	Buffer2+.6,Buffer2+.5
	movff	Buffer2+.7,Buffer2+.6
	movff	Buffer2+.8,Buffer2+.7
	movff	Buffer2+.9,Buffer2+.8
	movff	Buffer2+.10,Buffer2+.9
	movff	Buffer2+.11,Buffer2+.10
	movff	Buffer2+.12,Buffer2+.11
	movff	Buffer2+.13,Buffer2+.12
	movff	Buffer2+.14,Buffer2+.13
	movff	Buffer2+.15,Buffer2+.14
	decf	ColCount,f,access	; decrement shift count
	return

;***********************************************************************************;
tx.ascnum
	andlw	0x0f			; isolate digit
	iorlw	0x30
tx.byte
	btfss	TXSTA,TRMT
	bra		tx.byte			; if tx reg full
	movwf	TXREG
	return
;																				
print.rom					; move .16 bytes from TBLPTR to FSR0
	movlw	.16
go.pr
	tblrd*+
	movff	TABLAT,POSTINC0
	decfsz	WREG,f,access
	bra		go.pr
	return

;;;;
inimes:						; initial message: D-A-T-I 
	db	b'00011000',b'00010100'
	db	b'00010100',b'00010100'
	db	b'00011000',b'00000000'
	db	b'01000111',b'10100010'
	db	b'10100010',b'11100010'
	db	b'10100010',b'00001000'
	db	b'00001000',b'00001000'
	db	b'00001000',b'00001000'

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

id							; TX 3-digit or 4-digit ASCII serial number
							; including leading and trailing spaces
	clrf	KeyEdge,access

;	display serial number    MYserial ---> Buffer

	call	clrbuf
	lfsr	FSR0,Buffer+4

	movff	MYserial,Bin+0
	movff	MYserial+1,Bin+1
	call	p.bin			; print Bin (16-bit binary) on screen

;	infrared transmit

	movlw	' '
	call	tx.byte			; TX leading space

	movlw	'S'
	call	tx.byte			; TX S

	movlw	'N'
	call	tx.byte			; TX N

	movlw	':'
	call	tx.byte			; TX :

	movff	MYserial+0,Bin+0
	movff	MYserial+1,Bin+1
	call	bin2bcd			; convert serial number to BCD

	movf	Bcd+2,w,access
	andlw	0x0f			; isolate 10 thousands
	ifnz
	call	tx.ascnum		; TX 10 thousands

	swapf	Bcd+1,w,access
	andlw	0x0f			; isolate thousands
	bnz		p1000
	movf	Bcd+2,f,access
	ifnz
p1000
	call	tx.ascnum		; TX thousands

	movf	Bcd+1,w,access
	call	tx.ascnum		; TX hundreds

	swapf	Bcd+0,w,access
	call	tx.ascnum		; TX tens

	movf	Bcd+0,w,access
	call	tx.ascnum		; TX ones

	movlw	' '
	call	tx.byte			; TX trailing space

;	pause, then wait for key release

	movlw	.150				; wait about 700 ms
wait.id
	sync150
	decfsz	WREG,f,access
	bra		wait.id			; wait about 700 ms

;	wait for key release
wait.rel3
	incf	Rotor3,w,access
	bnz		wait.rel3		; wait for key release

	goto	farm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	#include <tetris.inc>	; tetris demo game
	#include <accel.inc>	; accelerator demo
	#include <display.inc>	; display demo
	#include <chargen.inc>	; character bitmaps, for display demo

	END


